/*+--------------------------------------------------------------------------+
 *| Copyright 1992 - 2002 Rational Software Corporation. All Rights Reserved.|
 *| This software contains proprietary and confidential information of       |
 *| Rational and its suppliers. Use, disclosure or reproduction is           |
 *| prohibited without the prior express written consent of Rational.        |
 *+--------------------------------------------------------------------------+
 */
/* 
//    The use of this system is subject to the software license terms and
//    conditions agreed upon between Rational and the Customer.
//
//                Copyright 1996 by Rational Software Corporation.
//
//                          RESTRICTED RIGHTS LEGEND
//
//    Use, duplication, or disclosure by the Government is subject to
//    restrictions as set forth in subdivision (b)(3)(ii) of the Rights in
//    Technical Data and Computer Software clause at 52.227-7013.
//
//
//                Rational Software Corporation
//                2800 San Tomas Expressway
//                Santa Clara, California 95051-0951
//
//   PROPRIETARY AND CONFIDENTIAL INFORMATION OF RATIONAL;
//   USE OR COPYING WITHOUT EXPRESS WRITTEN AUTHORIZATION
//   IS STRICTLY PROHIBITED.  THIS MATERIAL IS PROTECTED AS
//   AN UNPUBLISHED WORK UNDER THE U.S. COPYRIGHT ACT OF
//   1976.  CREATED 1996.  ALL RIGHTS RESERVED.
//
*/

#ifndef TMS_TMS_H
#define TMS_TMS_H

#include "testmate/tms_header_macros.h"
TMS_BEGIN_CPLUSPLUS

/* 
// The TMS package is the primary API for dynamically reporting
// test results to TestMate's servers, entering messages in the
// test log, registering assertions, etc.  Several queries are
// also provided to allow test drivers to obtain information on
// the state of the currently executing test list or test case.
//
// Calls to these routines while not executing under control of 
// TestMate will be ignored.
*/


#define TMS_TRUE 1
#define TMS_FALSE 0

/*
//----------------------------------------------------------------------
//
// Setting test results; getting prior test case results
//
// Test results are defined as strings, allowing for user-defined
// results (see User's Guide for information on defining new
// execution result values).  Functions are provided for the
// predefined execution results.
*/

/*
// The default test result kinds are exported by the following functions.
// The results are pointers to literal string and should not be freed.
*/
extern char * Tms_Pass ();
extern char * Tms_Fail ();

/*
// The following results are automatically set by TestMate, and need
// not be explicitly set by user test drivers.
*/

extern char * Tms_Failed_Assertion ();
extern char * Tms_Failed_Precondition ();
extern char * Tms_Fail_With_Exception ();
extern char * Tms_Terminated ();
extern char * Tms_Timeout_Elapsed ();
extern char * Tms_Unhandled_Signal ();


/*
// Register Test Results
//
// Two functions are provided.  The first registers a result for the
// currently executing test case.  The second allows you to register
// a test result for a test case (within the current test run) that
// already completed.
//
// Calls to these procedures while not under control of TestMate will
// be ignored, with no status set.
//
// Usage example:
//
//     if ( Device.Power_Up_Test_OK == 0 )
//         Tms_Register_Result( Tms_Pass() );
//     else
//         Tms_Register_Result( Tms_Fail() );
//
// Usage example of registering a user-defined test result:
//
//    Tms_Register_Result( "BIT_PASSED" );
//
// See the User's manual for more information regarding user-defined
// test results.
*/

extern void Tms_Register_Result (char *The_Result);
extern void Tms_Register_Specific_Result 
               (char *The_Result, char *For_The_Test_Case);



/*
//----------------------------------------------------------------------
// Assertions
//
// Assertions provide an alternative mechanism for registering test
// results.  Rather than explicitly calling Register_Result, a test
// may make repeated assertions (boolean conditions that should hold
// true) during a test.  If any of these assertions fail (i.e. are
// evaluated as false) in a particular test case, then TestMate will
// automatically register a Tms_Failed_Assertion result.
//
// Failed assertions do NOT terminate a test prematurely.
//
//
// If Assert is called at least once within a test case, and an evaluation
// action is not specified in the test case, then an implicit
// Tms_Register_Result will be performed as the default evaluation action.
//
// TestMate will make an entry in the log for failed assertions, by defalt.
// To change the behavior of assertion logging, select a behavior by 
// calling Tms_Assertion_Log_Control:
//
//    TMS_LPG_FAILURES_ONLY causes only failed assertions to be logged
//    TMS_LOG_NONE causes no assertions to be logged
//    TMS_LOG_ALL causes all assertions to be logged
//
// IMPORTANT NOTE:
// If your test case has no evaluation action, then TestMate
// evaluates all assertions.  If any assertion failed (i.e. evaluated
// to False) then TestMate will perform an implicit call to
// Tms_Register_Result (Tms_Failed_Assertion).  However, if you
// provide an evaluation action in the test case, then TestMate
// will perform no such implicit result evaluation -- it is up to
// the evaluation action to check the status of the assertions
// and any other criteria necessary to register a result
// explicitly.  The routinesTms_Check_Assertions is provided
// for this purpose.
//
// Usage example:
//
//    Tms_Assert( Device.Stat == 0, "Ensure device stat is 0");
//
// Usage example:
//
//    Tms_Assertion_Log_Control( Tms_Log_All );
//
//    Retval = Device.Status_Register1;
//    Tms_Assert( Retval == 0, "Ensure device status reg1 is zero");
//    Retval = Device.Status_Register2;
//    Tms_Assert( Retval != 0, "Ensure device status reg2 is non-zero");
//
// Usage example for C++ bools:
//
//    Tms_Assert( true, "C++ bool can be used in place of int");
// 
*/

extern void Tms_Assert ( int Condition, char *Description );

typedef enum {  TMS_LOG_ALL = 0, 
		TMS_LOG_FAILURES_ONLY = 1,
		TMS_LOG_NONE = 2 } Tms_Log_Mode;
		
extern void Tms_Assertion_Log_Control( Tms_Log_Mode Log_These_Entries );
		
/*
// Check assertion status
//
// Returns TMS_NONE if no assertions were made during this test case,
// TMS_ALL_TRUE if all assertions evaluated to true, and TMS_SOME_FALSE if
// at least one assertion failed.  Intended usage is within the
// test case evaluation action.
//
// Usage example:
//
//    void My_Eval_Action() 
//    {
//        if ( Tms_Assertion_Status == TMS_SOME_FALSE )
//            Tms_Register_Result(Tms_Failed_Assertion);
//        else {
//            Tms_Puts("Assertions OK; checking temperature");
//            if (Temperature->OK) then
//                Tms_Register_Result( Tms_Pass() );
//            else
//                Tms_Register_Result( Tms_Fail() );
//        }
//    }
*/
typedef enum {  TMS_NONE = 0, 
		TMS_ALL_TRUE = 1,
		TMS_SOME_FALSE = 2 } Tms_Assertion_Status;

extern Tms_Assertion_Status Tms_Check_Assertions ();


/*
//----------------------------------------------------------------------
//
// Access to the execution log output.
//
//
// These routines are supplied to allow access to the test results log.
//
// The test message log can be redirected independently of the
// test's stdout stream, allowing you to separate informational
// messages from the output of the unit or program under test.
//
// The TestMate switch TM_OUTPUT_FILTER controls which message kinds
// are entered into the log.  The TestMate switch TM_LOG_TIMESTAMPS
// controls whether or not the logged messages are prefixed with
// a timestamp.
//
*/


typedef enum {
    Tms_Nill,          /* no message generated  */
    Tms_Auxiliary_Msg, /* ::: preceding message */
    Tms_Debug_Msg,     /* ??? preceding message */
    Tms_Note_Msg,      /* //- preceding message */
    Tms_Positive_Msg,  /* +++ preceding message */
    Tms_Position_Msg,  /* >>> preceding message */
    Tms_Negative_Msg,  /* ++* preceding message */
    Tms_Warning_Msg,   /* !!! preceding message */
    Tms_Error_Msg,     /* *** preceding message */
    Tms_Sharp_Msg,     /* ### preceding message */
    Tms_Dollar_Msg,    /* $$$ preceding message */
    Tms_At_Msg,        /* @@@ preceding message */
    Tms_Text_Msg}      /*     unadorned message */
Tms_Log_Msg_Kind;



/*
// Enters Message into the test log.
//
// Usage example:
//
//    Tms_Puts("Starting test driver", Tms_Note_Msg);
//
// Usage example:
//
//    Tms_Puts("Could not setup unit for testing", Tms_Error_Msg);
//    Tms_Register_Result( Tms_Fail() );
*/

extern void Tms_Puts (char *Message, Tms_Log_Msg_Kind Msg_Kind);

/*
// Tms_Printf is the same as printf, except with an implicit '\n' at 
// the end of Fmt, and output is directed to the test log     
*/

extern void Tms_Printf (char *Fmt, Tms_Log_Msg_Kind Msg_Kind, ...);


/*
//----------------------------------------------------------------------
//
// Placeholder resolution
//
// Allows user's code to resolve placeholder specifiers, e.g.
// <test_case:t:r>.  For a complete description of placeholder syntax,
// and for a list of all placeholders and modifiers, see the User's Guide
//
// Result must be allocated by the caller.  If Result is too small
// too hold the resultant string, memory may be corrupted.  The function
// Tms_Resolve_Placeholders_n is provided to allow you to specify a
// maximum numbers to overwrite (including trailing null character)
//
// The parameter In_The_String is copied to Result with placeholders 
// resolved.  If the placeholders could not be resolved, then 
// that portion of the string is left unchanged.  The return value
// of the function is Result.
//
// Usage example:
//
//    char buffer[200];
//    Tms_Resolve_Placeholders("<user>", buffer);
*/

extern char *Tms_Resolve_Placeholders (char *In_The_String, char *Result);
extern char *Tms_Resolve_Placeholders_n (char *In_The_String, char *Result, int len);


/*
//----------------------------------------------------------------------
//
// Access to switches (from switches files, command line, test list)
*/


/*
// Returns the value of a switch.  All switches are interpreted as
// strings; conversion to boolean, integer, and enumeration
// switch types is not performed.
//
// See the user's guide for a list of all TestMate switches.
// You may also run the dialog TestMate:Reports:Switches, or
// run "testmate show_switches" from the command line.
//
// Result must be allocated by the caller.  If Result is too small
// too hold the resultant string, memory may be corrupted.  The function
// Tms_Value_n is provided to allow you to specify a limit to the
// number of characters to overwrite in Result (including trailing
// null character).
//
// Usage example:
//  
//    char buffer[200];
//    Tms_Value( "TM_INPUT_LOC", buffer );
//
*/

extern char *Tms_Value (char *The_Switch, char *Result);
extern char *Tms_Value_n (char *The_Switch, char *Result, int len);


/*
//----------------------------------------------------------------------
//
// Access to various run-time values of the current test's results
//
*/

/*
// Returns the number of tests that have completed with the specified
// Test_Result kind.
*/

extern int Tms_Number_Of_Tests (char *With_Result);


/*
// Returns the result of the last test case to complete processing.  The
// value returned will always be Result.  Result will point to a null 
// string if the value can not be retrieved.
//
// Caller must allocate Result, and it must be large enough to hold any
// anticipate result string.
*/

extern char *Tms_Prior_Test_Case_Result (char *Result);



/*
// This is a case insensitive equal.
//
// 0 (bool false) means that the strings were not equal; non-zero 
// (bool true) means that the strings are equal
//
// Usage example:
//
//     bool my_precondition() 
//     {
//         char *prior_result = 0;
//         Tms_Prior_Test_Case_Result(prior_result);
// 
//         if (prior_result)
//             return Tms_Is_Equal( prior_result, Tms_Pass() );
//         else
//             return false;
//     }    
*/

extern int Tms_Is_Equal (char *left, char *right);


/*
// Returns the test result of the currently active test case.  Note
// that if no result has yet been registered, this will return
// Unknown.  The value returned will always be the Result parameter.
// Result will be updated to the last registered result or Unknown.
*/

extern char *Tms_The_Test_Case_Result (char *Of_Test_Case, char *Result);



/*
//----------------------------------------------------------------------
//
// Exporting UNIX environment to subsequent executables
//
// If environment variables need to be passed from one executable
// process to another during a test run (e.g. from the test list prolog
// to all test drivers), then this routine needs to be called from
// within the exporter, before exiting.  In the example just given,
// the test list prolog would call this routine prior to exiting,
// and the environment variables would be exported for use later
// by the test drivers.
*/

extern void Tms_Register_Environment ();



TMS_END_CPLUSPLUS
#endif
